#!/usr/bin/env python
from jinja2 import Template, Environment, FileSystemLoader
import itertools
import subprocess
import datetime
import os
import sys
import time
import socket
from subprocess import call
import re
import json

def grab_card_ids():
    # This method runs nvidia-smi to grab the card ids, then returns a list
    
    if not os.path.isfile("/bin/nvidia-smi"):
        print("nvidia-smi binary not found!")
        exit(1)

    cmd = ["/bin/nvidia-smi", "--query-gpu=pci.bus_id","--format=csv,noheader"]
    p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    
    cards = []

    for line in p.stdout.readlines():
        bus_id_parts = line.rstrip().split(":")
        bus = int(bus_id_parts[1],16)
        dev_func = bus_id_parts[2].split(".")
        device = int(dev_func[0],16)
        function = int(dev_func[1],16)
        card = "PCI:{}:{}:{}".format(str(bus),str(device),str(function))
        cards.append(card)
    return cards

def grab_card_boardname():
    if not os.path.isfile("/bin/nvidia-smi"):
         print("nvidia-smi binary not found!")
         exit(1)

    cmd = ["/bin/nvidia-smi", "--query-gpu=name","--format=csv,noheader"]
    cards = []
    p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    for line in p.stdout.readlines():
        line = line.rstrip()
        cards.append(line)
    return cards

def write_xorg_conf(cards):
    num_of_cards = len(cards) + 1
    boardname = (grab_card_boardname())[0]
    gpus = []
    file_loader = FileSystemLoader('/opt/generate-xorg/template')
    env = Environment(loader=file_loader)
    template = env.get_template('xorg.conf.j2')

    for i in range(1, num_of_cards):
        monitors = []
        screens = []
        res = list(itertools.combinations(cards,i))

        for j in range(i):
            monitors.append("Monitor"+str(j))
            screens.append("Screen"+str(j))

        for card in res:
            filename = "/etc/X11/xorg.conf." + str(i) + str(res.index(card))
            template.stream({'boardname':boardname,'monitors':monitors,'screens':screens,'devices':card}).dump(filename)

if __name__ == "__main__":

    cards = grab_card_ids()
    write_xorg_conf(cards)


